#!/usr/bin/python
__author__ = 'Pavel Yarmak'

# IMPORTANT: For the script to function you must install pickledb: sudo pip install pickledb
scriptOptions = {
'daemonOptions': {
    'host': '127.0.0.1',
    'port': 58846,
    'username': 'admin',
    'password': 'EXAMPLE123'
},
'whitelist': {'Movies': 1, 'TV': 2},  # Only process torrents with these labels in the form of a dictionary with {'label': plex_section_code} format
'growlNotifications': True,  # To enable growl notifications set this to True and install gntp using pip:  sudo pip install gntp
'growlPassword': '',  # The password you have set in growl preferences after enabling network notifications (currently single password for all users)
'defaultHost': 'PAVEL-PC',  # The default host to which growl notifications will be sent (in case of torrents added using things such as RSS)
'pushoverNotifications': True,  # To enable pushover notifications set this to True and install pushnotify using pip:  sudo pip install pushnotify
'devKey': 'EXAMPLE123',  # Pushover app API Token/Key
'userKeys': ['EXAMPLE123'],  # List of Pushover User Keys
'appName': 'Deluge Alerts',  # The name of your pushover app
'pms_path': '/usr/lib/plexmediaserver'  # Path to directory where Plex Media Scanner is located
}

import sys
import subprocess
import logging
from os import path
from deluge.ui.client import client
from twisted.internet import reactor
import pickledb
import extractor
import pushnotify
import gntp.notifier
from deluge.common import windows_check


tid = sys.argv[1]
tname = sys.argv[2]
tpath = sys.argv[3]
host = None
growl = None
pushClient = None


def notify(title, description, sticky, icon="/pms.png"):
    def growl_register():
        # register with growl
        global growl
        if scriptOptions['growlNotifications']:
            logging.debug('Registering with growl')
            growl = gntp.notifier.GrowlNotifier(
                applicationName="DelugeGrowl",
                notifications=["New Messages"],
                defaultNotifications=["New Messages"],
                hostname=host,  # Defaults to localhost
                password=scriptOptions['growlPassword']  # Defaults to a blank password
            )
            growl.register()

    if not growl:
        try:
            growl_register()
        except gntp.errors.NetworkError as e:
            logging.error("failed to register with growl: %s" % e)
            return False
    logging.debug('Growl Notification: %s' % title)
    if scriptOptions['growlNotifications']:
        logging.debug('Growl Notification: %s' % title)
        growl.notify(
            noteType="New Messages",
            title=title,
            description=description,
            icon=file(path.dirname(path.abspath(sys.argv[0])) + icon).read(),
            sticky=sticky,
            priority=0,
        )
    return True


def notify_pushover(event, desc):
    logging.debug("Sending push notification to pushover")
    if scriptOptions['pushoverNotifications']:
        try:
            pushClient.notify(desc, event, split=True)
        except pushnotify.exceptions.PushNotifyError as e:
            logging.error("failed to push notification through pushnotify: %s" % e)


# call plex media scanner
def call_pms(label):
    exitcode = None
    logging.info("Starting Plex Media Scanner")

    if windows_check():
        cmd = scriptOptions['pms_path'] + "/Plex\ Media\ Scanner -s -c %s'" % scriptOptions['whitelist'][label]
    else:
        cmd = "sudo /bin/su plex -c 'env LD_LIBRARY_PATH=" + scriptOptions['pms_path'] + ' ' + scriptOptions['pms_path'] + "/Plex\ Media\ Scanner -s -c %s'" % scriptOptions['whitelist'][label]
    p = subprocess.Popen(cmd, stderr=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)
    while True:
        line = p.stderr.readline()
        out = p.stdout.readline()
        exitcode = p.poll()
        if (not out) and (not line) and (exitcode is not None):
            break
        line = line[:-1]
        out = out[:-1]
        if out != '':
            logging.info("%s", out)
        if line != '':
            logging.error("%s", line)
    if exitcode == 0:
        logging.info('Plex Media Scanner finished with success')
        notify("Ready to consume", tname, False)
        notify_pushover("Ready to consume", '%s has finished and been scanned by Plex' % tname)
    else:
        logging.error(cmd)
        notify("Problems with Plex Media Scanner", tname, False)
        notify_pushover("Problems with Plex Media Scanner", 'Plex had trouble scanning %s' % tname)
    return disconnect()


# get label from deluge
def on_connect_success(result):
    logging.debug('Connected to Deluge')
    return client.core.get_torrent_status(tid, ["labelplus_name", "move_completed"]).addCallback(on_get_torrent_status)


def on_connect_fail(result):
    logging.error("Connection failed: %s" % result)


def on_get_torrent_status(torrent):
    def on_get_torrent_status_loop(torrent):
        move_completed = torrent["move_completed"]
        if not move_completed:
            call_pms(label)
        else:
            return client.core.get_torrent_status(tid, ["move_completed"]).addCallback(on_get_torrent_status_loop)

    # Don't call plex media scanner if label not in whitelist
    label = torrent["labelplus_name"]
    move_complete = torrent["move_completed"]
    if label not in scriptOptions['whitelist'].keys():
        logging.info("Not a movie or tv show, nothing to do here.")
        notify("Torrent is ready for use", tname, False, '/deluge.png')
        notify_pushover("Torrent is ready for use", '%s has finished downloading and beeing processed' % tname)
        return disconnect()
    elif not move_complete:
        call_pms(label)
    else:
        logging.info("Waiting for the torrent to finish being moved")
        return client.core.get_torrent_status(tid, ["move_completed"]).addCallback(on_get_torrent_status_loop)


# close deluge connection
def disconnect(result=0):
    logging.debug("Disconnecting from Deluge")
    print "disconnect"
    return client.disconnect().addCallback(stop_reactor)


def stop_reactor(result=0):
    logging.debug("Stoping reactor")
    print "stop reactor"
    return reactor.stop()


def main():
    global host, pushClient

    if tid == 'test':
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s', filename=path.abspath(sys.argv[0]).replace('.py', '.log'))

    if tid != 'test':
        db = pickledb.load('/var/lib/deluge/delugeScripts/torrents.db', False)
        host = db.get(tid)
        if host:
            db.rem(tid)
            db.dump()

    if not host:
        host = scriptOptions['defaultHost']

    logging.info("Download complete: %s at %s/%s" % (tid, tpath, tname))
    extractor.on_torrent_finished(tid, tname, tpath)

    if tid != 'test':  # if supplied id is test, just call plex media scanner
        pushClient = pushnotify.get_client('pushover', scriptOptions['devKey'], scriptOptions['appName'])
        for key in scriptOptions['userKeys']:
            pushClient.add_key(key)
        logging.info("Connecting to deluge")
        d = client.connect(host=scriptOptions['daemonOptions']['host'], port=scriptOptions['daemonOptions']['port'],
                           username=scriptOptions['daemonOptions']['username'], password=scriptOptions['daemonOptions']['password'])
        d.addCallback(on_connect_success)
        d.addErrback(on_connect_fail)
        reactor.run()
    else:
        call_pms('Movies')


if __name__ == '__main__':
    main()