__author__ = 'Pavel Yarmak'

import os
import subprocess
import logging as log
from deluge.common import windows_check

PREFS = {
    "extract_path": "",
    "use_name_folder": False
}

def which(program):
    # Author Credit: Jay @ http://stackoverflow.com/a/377028
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None

if windows_check():
    win_7z_exes = [
        '7z.exe',
        'C:\\Program Files\\7-Zip\\7z.exe',
        'C:\\Program Files (x86)\\7-Zip\\7z.exe',
        ]
    switch_7z = "x -y"
    ## Future suport:
    ## 7-zip cannot extract tar.* with single command.
    #    ".tar.gz", ".tgz",
    #    ".tar.bz2", ".tbz",
    #    ".tar.lzma", ".tlz",
    #    ".tar.xz", ".txz",
    exts_7z = [
        ".rar", ".zip", ".tar",
        ".7z", ".xz", ".lzma",
        ]
    for win_7z_exe in win_7z_exes:
        if which(win_7z_exe):
            EXTRACT_COMMANDS = dict.fromkeys(exts_7z, [win_7z_exe, switch_7z])
            break
else:
    required_cmds=["unrar", "unzip", "tar", "unxz", "unlzma", "7zr", "bunzip2"]
    ## Possible future suport:
    # gunzip: gz (cmd will delete original archive)
    ## the following do not extract to dest dir
    # ".xz": ["xz", "-d --keep"],
    # ".lzma": ["xz", "-d --format=lzma --keep"],
    # ".bz2": ["bzip2", "-d --keep"],

    EXTRACT_COMMANDS = {
        ".rar": ["unrar", "x -o+ -y"],
        ".tar": ["tar", "-xf"],
        ".zip": ["unzip", ""],
        ".tar.gz": ["tar", "-xzf"], ".tgz": ["tar", "-xzf"],
        ".tar.bz2": ["tar", "-xjf"], ".tbz": ["tar", "-xjf"],
        ".tar.lzma": ["tar", "--lzma -xf"], ".tlz": ["tar", "--lzma -xf"],
        ".tar.xz": ["tar", "--xz -xf"], ".txz": ["tar", "--xz -xf"],
        ".7z": ["7zr", "x"],
        }
    # Test command exists and if not, remove.
    for cmd in required_cmds:
        if not which(cmd):
            for k,v in EXTRACT_COMMANDS.items():
                if cmd in v[0]:
                    log.warning("EXTRACTOR: %s not found, disabling support for %s", cmd, k)
                    del EXTRACT_COMMANDS[k]

if not EXTRACT_COMMANDS:
    log.error("EXTRACTOR: No archive extracting programs found")


def on_torrent_finished(torrent_id, name, path):
    """
    This is called when a torrent finishes and checks if any files to extract.
    """
    log.info("Checking if any files need to be extracted...")
    
    full_path = path + '/' + name
    try:
        os.chdir(full_path)
    except OSError as e:
        log.error("EXTRACTOR: Error looking up the filepath: %s" % e)

    for f in os.listdir("."):
        file_root, file_ext = os.path.splitext(f)
        file_ext_sec = os.path.splitext(file_root)[1]
        log.debug("EXTRACTOR: f = %s\n file_root = %s\n file_ext = %s\n file_ext_sec = %s" % (f, file_root, file_ext, file_ext_sec))
        if file_ext_sec and file_ext_sec + file_ext in EXTRACT_COMMANDS:
            file_ext = file_ext_sec + file_ext
        elif file_ext not in EXTRACT_COMMANDS or file_ext_sec == '.tar':
            log.debug("EXTRACTOR: Can't extract file %s of unknown type: %s" % (file_root, file_ext_sec))
            continue
        cmd = EXTRACT_COMMANDS[file_ext]

        # Now that we have the cmd, lets run it to extract the files
        fpath = os.path.normpath(os.path.join(full_path, f))

        # Get the destination path
        if not PREFS['extract_path']:
            dest = os.path.normpath(full_path)
        else:
            dest = os.path.normpath(PREFS["extract_path"])
        if PREFS["use_name_folder"]:
            dest = os.path.join(dest, name)

        # Create the destination folder if it doesn't exist
        if not os.path.exists(dest):
            try:
                os.makedirs(dest)
            except Exception, e:
                log.error("EXTRACTOR: Error creating destination folder: %s", e)
                return


        # Run the command and add some callbacks
        log.info("EXTRACTOR: Extracting %s to %s", fpath, dest)
        log.debug("EXTRACTOR: Extracting %s with %s %s to %s", fpath, cmd[0], cmd[1], dest)
        errorCode = subprocess.call(' '.join([cmd[0], cmd[1], str(fpath), str(dest)]), shell=True)
        if errorCode == 0:
            log.info("EXTRACTOR: Extract successful: %s (%s)", fpath, torrent_id)
        else:
            log.error("EXTRACTOR: Extract failed: %s (%s)", fpath, torrent_id)