#!/usr/bin/python
__author__ = 'Pavel Yarmak'

# IMPORTANT: For the script to function you must install pickledb: sudo pip install pickledb
scriptOptions = {
'daemonOptions': {
    'host': '127.0.0.1',
    'port': 58846,
    'username': 'admin',
    'password': 'EXAMPLE123'
},
'knownHosts': ['PAVEL-PC', 'TOM-PC'],  # Known LAN host names corresponding to the names of auto-add folders from which torrents are added
'growlNotifications': True,  # To enable growl notifications set this to True and install gntp using pip:  sudo pip install gntp
'growlPassword': '',  # The password you have set in growl preferences after enabling network notifications (currently single password for all users)
'defaultHost': 'PAVEL-PC',  # The default host to which growl notifications will be sent (in case of torrents added using things such as RSS)
'pushoverNotifications': True,  # To enable pushover notifications set this to True and install pushnotify using pip:  sudo pip install pushnotify
'devKey': 'EXAMPLE123',  # Pushover app API Token/Key
'userKeys': ['EXAMPLE123'],  # List of Pushover User Keys
'appName': 'Deluge Alerts',  # The name of your pushover app
}

import gntp.notifier
import sys
from os import path
import logging
from deluge.ui.client import client
from twisted.internet import reactor
import pickledb
import pushnotify


tid = sys.argv[1]
tname = sys.argv[2]
tpath = sys.argv[3]
growl = None
pushClient = None


# get label from deluge
def on_connect_success(result):
    logging.info('Connected to Deluge')
    return client.core.get_torrent_status(tid, ["save_path"]).addCallback(on_get_torrent_status)


def on_connect_fail(result):
    logging.error("Connection failed: %s" % result)


def on_get_torrent_status(torrent):
    hostname = path.basename(torrent["save_path"])
    if hostname in scriptOptions['knownHosts']:
        notify_growl(hostname)
        notify_pushover('Torrent started downloading', '%s has begun downloading' % tname)
        db = pickledb.load('/var/lib/deluge/delugeScripts/torrents.db', False)
        db.set(str(tid), str(hostname))
        db.dump()
        return disconnect()
    notify_growl(scriptOptions['defaultHost'])
    notify_pushover('Torrent started downloading', '%s has begun downloading' % tname)
    return disconnect()


# close deluge connection
def disconnect(result=0):
    logging.debug("Disconnecting from Deluge")
    print "disconnect"
    return client.disconnect().addCallback(stop_reactor)


def stop_reactor(result=0):
    logging.debug("Stoping reactor")
    print "stop reactor"
    return reactor.stop()   


def notify_growl(growlHostname):
    def growl_register():
        global growl
        #register with growl
        growl = gntp.notifier.GrowlNotifier(
            applicationName="DelugeGrowl",
            notifications=["New Messages"],
            defaultNotifications=["New Messages"],
            hostname=growlHostname,  # Defaults to localhost
            password=""  # Defaults to a blank password
        )
        growl.register()

    if not growl:
        try:
            growl_register()
        except gntp.errors.NetworkError as e:
            logging.error("failed to register with growl: %s" % e)
            return False
    #notify of download start
    growl.notify(
        noteType="New Messages",
        title="Downloading",
        description=tname,
        icon=file(path.dirname(path.abspath(sys.argv[0])) + '/deluge.png').read(),
        sticky=False,
        priority=0,
    )
    return True


def notify_pushover(event, desc):
    logging.debug("Sending push notification to pushover")
    if scriptOptions['pushoverNotifications']:
        try:
            pushClient.notify(desc, event, split=True)
        except pushnotify.exceptions.PushNotifyError as e:
            logging.error("failed to push notification through pushnotify: %s" % e)


def main():
    global pushClient
    
    if tid == 'test':
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(message)s', filename=path.abspath(sys.argv[0]).replace('.py', '.log'))
    
    logging.info("Download started: %s of %s" % (tid, tname))
    logging.info("Connecting to deluge")
    pushClient = pushnotify.get_client('pushover', scriptOptions['devKey'], scriptOptions['appName'])
    for key in scriptOptions['userKeys']:
        pushClient.add_key(key)
    d = client.connect(host=scriptOptions['daemonOptions']['host'], port=scriptOptions['daemonOptions']['port'],
                       username=scriptOptions['daemonOptions']['username'], password=scriptOptions['daemonOptions']['password'])
    d.addCallback(on_connect_success)
    d.addErrback(on_connect_fail)
    reactor.run()

if __name__ == '__main__':
    main()